//Written by Minghao Wei for Technologic System
//June 3rd, 2016
//The way to use this API is similar to use Object Oriented programming
//First you need to use adcSetup to setup the pointer to the adc
//after that, you just need that pointer to access all the functions
//Please see the example test code for more detail of how to use this API
#include <stdio.h>
#include <stdint.h>
#include <assert.h>
#include <unistd.h>
#include <errno.h>
#include <sys/mman.h>
#include <fcntl.h>
#include <sys/stat.h>

#include "adc24.h"

//Setup pointer to access BASE of adc24
//input: adc16_base (depend on the jumper states)
//PC104BASE: your SBC's pc104 I/O address
//most of the print statement is for debugging purpose, not necessary for the API
volatile uint16_t*adcSetup(uint16_t adc16_base, uint32_t PC104BASE){
        int pc104id =0;
        int mem = 0;
        volatile uint16_t *adc24_io;
        volatile uint16_t *pc104_base = 0;
        //setup device pointer
        mem = open("/dev/mem", O_RDWR|O_SYNC);
        pc104_base = mmap(0, getpagesize(), PROT_READ | PROT_WRITE, MAP_SHARED,mem, PC104BASE);
        adc24_io = pc104_base + (adc16_base/2);
        //printf("returning pointer to address 0x%x\n", adc16_io);
        return adc24_io;
}

//start state machine, channelNum can be between 0-5
void adcStart(volatile uint16_t* adc24_io, int channelNum){
        uint16_t startOption = 0x161 | ((channelNum << 1) & 0xf);
        printf("start option 0x%x\n", startOption);
        adc24_io[2/2] = startOption;
	//bit 0 -> 1
	//bit 4-1 -> channelNum
	//bit 5 & 8 -> 1
	//bit 7 -6 -> 01 (0 to +2xVref(default))
}

//stop state machine
void adcStop(volatile uint16_t* adc24_io){
        adc24_io[2/2] = adc24_io[2/2] & 0xfffe;
        usleep(50);//you may not need this
}

//set 24-bit pacing clock counter
void setClockSpeed(volatile uint16_t* adc24_io, uint32_t speed){
        uint16_t lsb = 0;
        uint16_t msb = 0;
        lsb = (uint16_t)((uint32_t)(speed & 0x0000ffff));
        msb = (uint16_t)((uint32_t)(speed & 0xffff0000) >> 16);
        adc24_io[4/2] = msb;//register 4
        adc24_io[6/2] = lsb;//register 6
}

void cleanFIFO(volatile uint16_t* adc24_io){
        int temp;
        unsigned int sample_count = adc24_io[0x8/2] >> 6;//debug
        printf("FIFO count : %d\n", sample_count);//debug
        while((adc24_io[0x8/2] >> 6) != 0x0){
                temp = adc24_io[0xa/2];
                fflush(stdout);
                usleep(2);//little delay is necessary for clean FIFO
        }
        //printf("test\n");
        if((adc24_io[0x8/2] >> 6) != 0x0){//check to see is FIFO empty
                printf("Error cleaning FIFO, %d did not clean\n", (adc24_io[0x8/2]) >> 6);
        }//if you get into this loop, try increase the delay
}

//set interrupt level (max 512), default 256 (half full)
//Please see adc24 manual for more information about register 0x8
void setInterrupt(volatile uint16_t* adc24_io, uint16_t level){
        uint16_t interrupt = (level << 6) | 0x1;
        //printf("0x%x\n", interrupt);
        adc24_io[0x8/2] = interrupt;
}

//display board information to the console
//Please see adc24 manual for more information about register 0x0
void getBoardInfor(volatile uint16_t* adc24_io){
        uint8_t id = adc24_io[0] & 0x00ff;//board ID, register 0x0 bit 7-0
        uint8_t PLDRevision = (adc24_io[0] >> 8) & 0x0f;//bit 11-8
        uint8_t jumperStatus = adc24_io[0] >> 12;//bit 15-12
        printf("ID: 0x%x\nPLD Revision: 0x%x\nJumper Status: 0x%x\n", id, PLDRevision, jumperStatus);
}

//return FIFO count
uint16_t getFIFOcount(volatile uint16_t* adc24_io){
        return adc24_io[0x8/2] >> 6;//register 0x8 bit 15-6
}

//return channel number of the first sample in FIFO
uint8_t getHeadChannel(volatile uint16_t* adc24_io){
        return (adc24_io[0x8/2] >> 1) & 0x1f;//register 0x8 bit 5-1
}

//return a struct that contain information of FIFO read
//Please see adc24 manual for more information about register 0xa
struct adcFIFO readFIFO(volatile uint16_t* adc24_io){
        struct adcFIFO read;
        uint16_t dataRead = adc24_io[0xa/2];
        read.data = dataRead & 0xfff;//bit 11-0
        read.channel = (dataRead >> 12) & 0x7;//bit 14-12
        read.chip = dataRead >> 15;//bit 15
        return read;
}

//return 16-bit data read from FIFO
uint16_t readFIFO_Data(volatile uint16_t* adc24_io){
        return adc24_io[0xa/2];
}
                