//Written by Minghao Wei for Technologic System
//3rd June, 2016
//This code will prove a simple API for users
//This code is for educational intent also
//-------------------------------------------------------------
//Implementation Note:
//This API is for TS-ADC24 use only
//This API will assume your SBC's PC104 is on, see your SBC's manual
//This API will assume you are using the 16-bit mode
//-------------------------------------------------------------
//Please see ADC24 manual for detail information about ADC24
//updating the pacing clock requires stopping the state machine
//FIFO can hold 512 sample, state machine stop when the FIFO is full
//restart the FIFO will clean all data in the FIFO, in other word
//The remaining data in the FIFO is available until the system restarted.
//The minimal acceptable value to be loaded for the pacing clock is hex 20, 1us
//24-bit pacing clock = 0 indicates continuous ADC scanning
//-------------------------------------------------------------
//These functions is very base and should be easy to understand
//Depend on your need, writting your own code should give you better effciency
//In order to synchronize the internal machine and initialize the ADC chips after power up,
//it is recommended to perform an initialization cycle(Reg.ADCCFG.SYSCOM) before read utilization
//
//Again, please see the ADC24 manual for all detail information
//-------------------------------------------------------------
#include <stdio.h>
#include <stdint.h>
#include <errno.h>
#include <sys/mman.h>

struct adcFIFO{//A struct that contain information read from FIFO
        uint8_t chip;//chip number, 0 or 1, bit 15
        uint8_t channel;//Channel number, bit 14-12
        uint16_t data;//12-bit data,bit 11-0
};

//Setup pointer to access the adc24, return pointer
//input: base address for adc, PC104 I/O address for your SBC
volatile uint16_t* adcSetup(uint16_t adc16_base, uint32_t PC104BASE);

//Start state machine with selected channel (0-5)
void adcStart(volatile uint16_t* adc24_io, int channelNum);

//Stop the state machine
void adcStop(volatile uint16_t* adc24_io);

//set your 24-bit pacing clock counter
//bigger number means slower FIFO fill up
//input 0 - 32000000, 32000000 = sample frequency in Hz
void setClockSpeed(volatile uint16_t* adc24_io, uint32_t speed);

//Clean the FIFO by reading everything off from it
//Stop state machine before using this function
void cleanFIFO(volatile uint16_t* adc24_io);

//set and enable interrupt level, default is FIFO half full
//which is 256. see more information about interrupt in manual
void setInterrupt(volatile uint16_t* adc24_io, uint16_t level);

//Display Board ID, PLD Revision, Jumper state to console
//mainly use for debugging purpose
void getBoardInfor(volatile uint16_t* adc24_io);

//return the current FIFO count in a 16-bit number
uint16_t getFIFOcount(volatile uint16_t* adc24_io);

//return the FIFO's next sample's channel number
//see more information about channel number in adc24 manual
uint8_t getHeadChannel(volatile uint16_t* adc24_io);

//read one sample from the FIFO
//return a struct that contain information of that sample
//chip number, channel number, 12-bit data read
struct adcFIFO readFIFO(volatile uint16_t* adc24_io);

//read one sample from the FIFO
//return a 16-bit number from register 0xA
//please see manual about information of register 0xA
//bit 15:chip number(0-chip 1, 1-chip 2)
//bit 14-12: current number of channel for the data read
//bit 11-0: 12-bit ADC data read
uint16_t readFIFO_Data(volatile uint16_t* adc24_io);
